/*
Copyright (C) 2009 Scott Grauer-Gray, Chandra Kambhamettu, and Kannappan Palaniappan

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
*/

//Defines the functions used to load the input images and store the disparity map image for use in the CUDA BP implementation

#include "imageHelpersHostHeader.cuh"


//functions used to load input images/save resulting movment images

//function to retrieve the disparity values from a disparity map with a known scale factor
__host__ float* retrieveDisparityValsFromStereoPGM(const char* filePathPgmImage, unsigned int widthImage, unsigned int heightImage, float scaleFactor)
{
	unsigned int* imageData = new unsigned int[widthImage*heightImage];
	unsigned int imageWidth = widthImage;
	unsigned int imageHeight = heightImage;

	cutLoadPGMi( filePathPgmImage, &imageData, &imageWidth, &imageHeight);

	float* disparityVals = new float[widthImage*heightImage];

	//go through every pixel and retrieve the Stereo value using the pixel value and the scale factor
	for (unsigned int pixelNum = 0; pixelNum < (widthImage*heightImage); pixelNum++)
	{
		disparityVals[pixelNum] = imageData[pixelNum] / scaleFactor;
	}

	delete [] imageData;

	return disparityVals;
}

//load the PGM image and return as an array of floats
__host__ unsigned int* loadImageFromPGM(const char* filePathPgmImage, unsigned int widthImage, unsigned int heightImage)
{
	unsigned int* imageData = new unsigned int[widthImage*heightImage];
	unsigned int imageWidth = widthImage;
	unsigned int imageHeight = heightImage;

	cutLoadPGMi( filePathPgmImage, &imageData, &imageWidth, &imageHeight);

	return imageData;
}

//save the calculated disparity map from image 1 to image 2 as a grayscale image using the SCALE_MOVEMENT factor with
//0 representing "zero" intensity and the intensity linearly increasing from there using SCALE_MOVEMENT
__host__ void saveDisparityImageToPGM(const char* filePathSaveImage, float scaleMovement, float*& calcDisparityBetweenImages, unsigned int widthImage, unsigned int heightImage)
{
	//declare and allocate the space for the movement image to save
	unsigned int* movementImageToSave = new unsigned int[widthImage*heightImage];

	//go though every value in the movementBetweenImages data and retrieve the intensity value to use in the resulting "movement image" where minMovementDirection
	//represents 0 intensity and the intensity increases linearly using scaleMovement from minMovementDirection
	for (unsigned int currentPixel = 0; currentPixel < (widthImage*heightImage); currentPixel++)
	{

		//add .5 and truncate to "round" the intensity to save to an integer
		movementImageToSave[currentPixel] = (int)((calcDisparityBetweenImages[currentPixel])*scaleMovement + .5f);
	}

	cutSavePGMi(filePathSaveImage, movementImageToSave, widthImage, heightImage);
}
